require 'arb/comm'

module ARb
  class ARbBlockThread < ARbThread
    def initialize(arbblock, *args, &block)
      prio = Thread.current.priority - arbblock.priority
      timer = arbblock.life
      super(prio, timer, {:ARbBlock, arbblock}, *args, &block)
    end
  end

  Comm = ARbComm.new
  Server = ARbServer.new(Comm)

  class ARbObject
    @@server = Server
    @@comm = Comm

    def initialize(name, obj = nil)
      @@server.regist(name, obj) if obj
      @name = name
    end

    def method_missing(msg_id, *a, &b)
      @@comm.request(@name, msg_id, a, b)
    end

    undef to_a
    undef to_s
  end

  class ARbBlock
    @@priority = -1
    @@life = 10
    @@magic = 100

    def ARbBlock.priority=(v)
      @@priority = v
    end
    
    def ARbBlock.life=(v)
      @@life = v
    end
    
    def ARbBlock.magic=(v)
      @@magic = v
    end
    
    def initialize(src, life=@@life, magic=@@magic, priority=@@priority)
      @src = src.dup.freeze
      @life = [life, @@life].min
      @magic = [magic, @@magic].min
      @priority = priority
      @thread = nil
    end
    attr_reader :life, :magic, :priority

    def run
      prio = Thread.current.priority + @@priority
      here = ARbObject.new('here')
      src = prepare_src
      here.taint
      @thread = ARbBlockThread.new(self, here) do |here|
	eval src
      end
    end

    def value
      @thread.thread.value if @thread
    end

    def kill
      @thread.kill if @thread
    end

    def pay(point)
      @magic -= point
      raise ARbNoAbilityError if @magic <= 0
    end
    
    private
    def prepare_src
      @src
    end
  end
end

