require 'arb/thread'

module ARb
  class ARbComm
    def initialize
      @que = Queue.new
      @waiting = {}.taint
      self.taint
    end

    def accept
      raise SecurityError, "Insecure operation `accept'" if $SAFE > 3
      @que.pop
    end
    
    def reply(msg, reply)
      raise SecurityError, "Insecure operation `reply'" if $SAFE > 3
      @waiting[msg.sender] = reply
      msg.sender.run
    end

    def request(recv, msg, arg, block)
      begin
	msg = Message.new(recv, msg, arg, block)
	@waiting[msg.sender] = nil
	@que.push(msg)
	while (@waiting[msg.sender].nil?)
	  Thread.stop
	end
	result = @waiting[msg.sender]
	result.recv
      ensure
	@waiting.delete(msg.sender)
      end
    end
  end

  class ARbReply
    def initialize(succ, result)
      @succ = succ
      @result = result.taint
      self.taint
    end

    def recv
      raise @result unless @succ
      @result
    end
  end

  class Message
    def initialize(receiver, msg, arg, block)
      @sender = Thread.current
      @receiver = receiver
      @msg = msg
      @arg = arg
      @block = block
    end
    attr_reader :sender, :receiver, :msg, :arg, :block

    def send(obj)
      succ = true
      begin
	pay
	if @block
	  result = obj.__send__(@msg, *arg, &@block)
	else
	  result = obj.__send__(@msg, *arg)
	end
      rescue Exception
	succ = false
	result = $!.taint
      end
      ARbReply.new(succ, result)
    end

    private
    def pay
      client = @sender[:ARbBlock]
      return unless client
      client.pay(1)
    end
  end

  class ARbServer
    def initialize(comm)
      @comm = comm
      @thread = main_thread
      @table = {}
    end
    attr_reader :thread

    def main_thread
      Thread.new do
	main_loop
      end
    end

    def regist(name, obj)
      raise ARbRuntimeError if @table.include? name
      @table[name] = obj
    end

    def name_to_obj(name)
      @table[name]
    end

    def main_loop
      loop do
	msg = @comm.accept
	Thread.new do 
	  reply = nil
	  begin
	    obj = name_to_obj(msg.receiver)
	    Thread.current[:ARb] = { 'message', msg }
	    reply = msg.send(obj)
	  ensure
	    reply = ARbReply.new(false, ARbRuntimeError.new) unless reply
	    @comm.reply(msg, reply)
	  end
	end
      end
    end
  end
end
