require 'thread'

module ARb
  class ARbError < RuntimeError; end
  class ARbRuntimeError < ARbError; end
  class ARbPermitError < ARbError; end
  class ARbTimeoutError < ARbPermitError; end
  class ARbNoAbilityError < ARbPermitError; end

  class ARbThread
    def initialize(prio=nil, timer=nil, vars={}, *args, &block)
      @grp = ThreadGroup.new
      keeper(timer) if timer
      @thread = runner(vars, prio, block, args)
    end
    attr_reader :thread

    def kill(timeout = false)
      Thread.exclusive do
	@grp.list.each do |th|
	  th.raise(ARbTimeoutError) if timeout
	  th.kill
	end
      end
    end

    def kill2(timeout = false)
      list = @grp.list
      while list.size > 0
	list.each do |th|
	  th.kill
	end
	list = @grp.list
      end
    end
    
    private
    def runner(vars, prio, block, args)
      Thread.new do
	vars.each do |k,v|
	  Thread.current[k] = v
	end
	safe4(prio, block, args)
      end
    end

    def safe4(prio, block, args)
      @grp.add Thread.current
      Thread.current.priority = prio if prio
      $SAFE = 4
      block.call(*args)
    end

    def keeper(sec)
      Thread.new do
	sleep sec
	kill(true)
      end
    end
  end
end
