require 'runit/testcase'
require 'runit/cui/testrunner'
require 'arb/block'

class Front
  def initialize
    @count = 1
  end

  def log(s)
    @count += 1
    puts s
    s * @count
  end

  def each
    [1, 2, 3].each do |x|
      yield x
    end
  end

  def ary(*arg)
    arg
  end

  def hello
    'hello'
  end
end

$front = Front.new
ARb::ARbObject.new('here', $front)

module T4Util
  def t4_thread_value(*arg, &block)
    th = t4_thread(*arg, &block)
    th.value
  end

  def t4_thread(*arg)
    th = Thread.new() do
      $SAFE = 4
      yield *arg
    end
  end
end

class ARbCommTest < RUNIT::TestCase
  include T4Util

  def test_00
    assert_exception(SecurityError) do
      t4_thread_value do
	ARb::Comm.accept
      end
    end
    assert_exception(SecurityError) do
      t4_thread_value do
	ARb::Comm.reply(nil, nil)
      end
    end
  end

  def test_01
    comm = ARb::ARbComm.new
    t4_thread(comm) do |comm|
      comm.request('test_01', :test, [], nil)
    end
    msg = comm.accept
    assert_equal(msg.receiver, 'test_01')
    assert_equal(msg.msg, :test)
    assert_equal(msg.arg, [])
    assert_equal(msg.block, nil)
  end
end

class ARbObjectTest < RUNIT::TestCase
  include T4Util

  def test_regist
    here = ARb::ARbObject.new('here')
    assert_equal(here.hello, 'hello')
  end

  def test_cannot_re_regist
    assert_exception(ARb::ARbRuntimeError) do
      ARb::ARbObject.new('here', self)
    end
  end

  def test_yield
    v = t4_thread_value do 
      here = ARb::ARbObject.new('here')
      sum = 0
      here.each do |s|
	sum += s
      end
      sum
    end
    assert_equal(v, 6)
  end

  def test_args
    v = t4_thread_value do
      here = ARb::ARbObject.new('here')
      here.ary(1, 2, 3)
    end
    assert_equal(v, [1,2,3])
  end

  def bar
    t = foo('hello')
    t.join
    p t.value

    p ARb::Comm
    
    block = ARb::ARbBlock.new('here.log("hi")', 100)
    block.run
    p block.value
  end
end

class ARbBlockTest < RUNIT::TestCase
  def test_00
    block = ARb::ARbBlock.new('here.hello', 1)
    block.run
    assert_equal(block.value, 'hello')
  end
end

if __FILE__ == $0
  RUNIT::CUI::TestRunner.run(ARbCommTest.suite)
  RUNIT::CUI::TestRunner.run(ARbObjectTest.suite)
  RUNIT::CUI::TestRunner.run(ARbBlockTest.suite)
end


